@@ -1,6 +1,6 @@
 (* camlp5r pa_macro.cmo q_MLast.cmo *)
-(* $Id: ast2pt.ml 1927 2009-05-23 09:04:13Z deraugla $ *)
-(* Copyright (c) INRIA 2007-2008 *)
+(* $Id: ast2pt.ml,v 1.69 2010/02/19 09:06:37 deraugla Exp $ *)
+(* Copyright (c) INRIA 2007-2010 *)
 
 open MLast;
 open Parsetree;
@@ -14,7 +14,14 @@ THEN
 END;
 
 IFDEF
-  OCAML_3_11 OR OCAML_3_11_0 OR OCAML_3_11_1 OR OCAML_3_12_0
+  OCAML_3_12_0
+THEN
+  DEFINE AFTER_OCAML_3_12
+END;
+
+IFDEF
+  OCAML_3_11 OR OCAML_3_11_0 OR OCAML_3_11_1 OR OCAML_3_11_2 OR
+  AFTER_OCAML_3_12
 THEN
   DEFINE AFTER_OCAML_3_11
 END;
@@ -465,7 +472,7 @@ value rec patt =
             [ <:patt< $uid:s$ >> ->
                 Ppat_construct (mkli (conv_con s) [i :: il]) None
                   (not no_constructors_arity.val)
-            | _ -> error (loc_of_patt p2) "uppercase identifier expected" ]
+            | _ -> error (loc_of_patt p2) "bad access pattern" ]
         | _ -> error (loc_of_patt p2) "bad pattern" ]
       in
       mkpat loc p
@@ -512,6 +519,12 @@ value rec patt =
   | PaInt loc _ _ -> error loc "special int not impl in patt"
   | PaFlo loc s -> mkpat loc (Ppat_constant (Const_float (uv s)))
   | PaLab loc _ _ -> error loc "labeled pattern not allowed here"
+  | PaLaz loc p ->
+      IFDEF AFTER_OCAML_3_11 THEN
+        mkpat loc (Ppat_lazy (patt p))
+      ELSE
+        error loc "lazy patterns not in this version"
+      END
   | PaLid loc s -> mkpat loc (Ppat_var (uv s))
   | PaOlb loc _ _ -> error loc "labeled pattern not allowed here"
   | PaOrp loc p1 p2 -> mkpat loc (Ppat_or (patt p1) (patt p2))
@@ -522,7 +535,12 @@ value rec patt =
           let c2 = char_of_char_token loc2 (uv c2) in
           mkrangepat loc c1 c2
       | _ -> error loc "range pattern allowed only for characters" ]
-  | PaRec loc lpl -> mkpat loc (Ppat_record (List.map mklabpat (uv lpl)))
+  | PaRec loc lpl ->
+      IFDEF AFTER_OCAML_3_12 THEN
+        mkpat loc (Ppat_record (List.map mklabpat (uv lpl)) Closed)
+      ELSE
+        mkpat loc (Ppat_record (List.map mklabpat (uv lpl)))
+      END
   | PaStr loc s ->
       mkpat loc
         (Ppat_constant (Const_string (string_of_string_token loc (uv s))))
